package com.zanxiang.sdk.service.Impl.pay;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.alipay.easysdk.factory.Factory;
import com.alipay.easysdk.kernel.Config;
import com.alipay.easysdk.kernel.util.ResponseChecker;
import com.alipay.easysdk.payment.app.models.AlipayTradeAppPayResponse;
import com.alipay.easysdk.payment.page.models.AlipayTradePagePayResponse;
import com.alipay.easysdk.payment.wap.models.AlipayTradeWapPayResponse;
import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.enums.ResEnum;
import com.zanxiang.common.utils.URIUtil;
import com.zanxiang.sdk.domain.bo.PlatformOrderBO;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.service.OrderPayService;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.*;

/**
 * @author xufeng
 * @date 2022/6/8 15:37
 */
@Component
@Slf4j
@Service
public class AlipayServiceImpl extends PayService implements OrderPayService {
    private static final Logger logger = LoggerFactory.getLogger(AlipayServiceImpl.class);

    private String returnUrl = "http://localhost/pay/notify";

    /**
     * 支付调起
     *
     * @param product
     * @return
     */
    @Override
    public ResultMap create(ProductPayParamBO product) {
        this.attach = product.getAttach();
        configInit(product.getConfig());
        Map<String, String> attachMap = JSON.parseObject(JSON.toJSONString(this.attach), new TypeReference<Map<String, String>>() {
        });
        this.returnUrl = URIUtil.fillUrlParams(this.returnUrl, attachMap, true);

        switch (product.getPayDevice()) {
            case 1:
                return this.pc(product);
            case 2:
                return this.mobile(product);
            case 3:
                return this.app(product);
            default:
                throw new RuntimeException("未知支付方式");
        }
    }

    /**
     * 异步回调
     *
     * @param request
     * @param response
     * @return
     */
    @Override
    public String notify(HttpServletRequest request, HttpServletResponse response) {
        try {
            Map<String, String> params = new HashMap<>();
            Enumeration<String> parameterNames = request.getParameterNames();
            while (parameterNames.hasMoreElements()) {
                String parameterName = parameterNames.nextElement();
                params.put(parameterName, request.getParameter(parameterName));
            }
            String outTradeNo = request.getParameter("out_trade_no");
            Boolean flag = Factory.Payment.Common().verifyNotify(params);
            if (flag) {
                PlatformOrderBO bo = new PlatformOrderBO();
                bo.setId(outTradeNo);
                bo.setUserId(request.getParameter("user_id"));
                bo.setRealAmount(new BigDecimal(request.getParameter("totalFee")));
                bo.setMerchantOrderNo(request.getParameter("trade_no"));
                bo.setStatus(OrderStateEnum.SUCCESS.getCode());
                bo.setGamePaywayId(request.getParameter("gamePayWayId"));
                bo.setPayTime(new Date());
                Boolean result = platformOrderService.pay(bo);
                if (result) {
                    logger.info("支付宝异步回调成功 request:{},商户订单号为：{}", request, outTradeNo);
                    return ResEnum.SUCCESS.getMsg();
                }
            }
            logger.error("支付宝异步回调-验证签名失败 request:{}", request);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResEnum.FAIL.getMsg();
    }

    /**
     * 同步回调
     *
     * @param request
     * @return
     */
    @Override
    public ResultMap synNotify(HttpServletRequest request) {
        try {
            //获取支付宝GET过来反馈信息
            Map<String, String> params = new HashMap<>();
            Map<String, String[]> requestParams = request.getParameterMap();
            for (Iterator<String> iter = requestParams.keySet().iterator(); iter.hasNext(); ) {
                String name = iter.next();
                String[] values = requestParams.get(name);
                String valueStr = "";
                for (int i = 0; i < values.length; i++) {
                    valueStr = (i == values.length - 1) ? valueStr + values[i]
                            : valueStr + values[i] + ",";
                }
                //乱码解决，这段代码在出现乱码时使用
                valueStr = new String(valueStr.getBytes("ISO-8859-1"), "utf-8");
                params.put(name, valueStr);
            }
            //商户订单号
            String orderNo = new String(request.getParameter("out_trade_no").getBytes("ISO-8859-1"), "UTF-8");
            //前台回调验证签名 v1 or v2
            Boolean flag = Factory.Payment.Common().verifyNotify(params);
            if (flag) {
                logger.info("订单号" + orderNo + "验证签名结果[成功].");
                //处理业务逻辑
            } else {
                logger.info("订单号" + orderNo + "验证签名结果[失败].");
            }
            return ResultMap.ok("success");
        } catch (Exception e) {
            e.printStackTrace();
            //处理异常信息
            return ResultMap.error(e.getMessage());
        }
        //支付成功、跳转到成功页面
        //return "success.html";
    }

    /**
     * 手机支付
     *
     * @param product
     * @return
     */
    private ResultMap mobile(ProductPayParamBO product) {
        try {
            logger.info("mobile端支付生成，请求参数 bo:{}", product);
            AlipayTradeWapPayResponse response = Factory.Payment
                    .Wap()
                    .pay(JSONObject.toJSONString(this.attach), product.getOutTradeNo(), product.getTotalFee(), "", this.returnUrl);
            logger.info("mobile端支付生成，result:{}", response);
            if (ResponseChecker.success(response)) {
                return ResultMap.ok(response.getBody());
            } else {
                return ResultMap.ok(ResEnum.FAIL.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResultMap.error(ResEnum.FAIL.getMsg());
        }
    }

    /**
     * 电脑支付
     *
     * @param product
     * @return
     */
    private ResultMap pc(ProductPayParamBO product) {
        try {
            logger.info("pc端支付生成，请求参数 bo:{}", product);
            AlipayTradePagePayResponse response = Factory.Payment
                    .Page()
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee(), this.returnUrl);
            logger.info("pc端支付生成，result:{}", response);
            if (ResponseChecker.success(response)) {
                return ResultMap.ok(response.getBody());
            } else {
                return ResultMap.error(ResEnum.FAIL.getMsg());
            }
        } catch (Exception e) {
            logger.error("pc端支付生成异常 e:{}", e);
            e.printStackTrace();
            return ResultMap.error(ResEnum.FAIL.getMsg());
        }
    }

    /**
     * app支付
     *
     * @param product
     * @return
     */
    private ResultMap app(ProductPayParamBO product) {
        try {
            AlipayTradeAppPayResponse response = Factory.Payment
                    .App()
                    .pay(product.getSubject(), product.getOutTradeNo(), product.getTotalFee());
            if (ResponseChecker.success(response)) {
                return ResultMap.ok(response.getBody());
            } else {
                return ResultMap.error(ResEnum.FAIL.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("app端支付生成异常 e:{}", e);
            return ResultMap.error(ResEnum.FAIL.getMsg());
        }
    }

    /**
     * 配置初始化
     */
    public void configInit(Object obj) {
        Config config = JSONObject.parseObject(obj.toString(), Config.class);
        Factory.setOptions(config);
    }

}
